@@ -60,23 +60,35 @@ class GenericDBListStore(gtk.GenericTreeModel):
     def _on_inserted(self, db, oid):
         self.oidcache = []
         offset = self._get_offset(oid)
-        rowref = self.get_iter(offset)
-        path = self.get_path(rowref)
-        self.row_inserted(path, rowref)
+        try:
+            rowref = self.get_iter(offset)
+            path = self.get_path(rowref)
+            self.row_inserted(path, rowref)
+        except ValueError:
+            #not a valid rowref
+            pass
                 
     def _on_modified(self, db, oid):
         self.oidcache = []
         offset = self._get_offset(oid)
-        rowref = self.get_iter(offset)
-        path = self.get_path(rowref)
-        self.row_changed(path, rowref)
+        try:
+            rowref = self.get_iter(offset)
+            path = self.get_path(rowref)
+            self.row_changed(path, rowref)
+        except ValueError:
+            #not a valid rowref
+            pass
         
     def _on_deleted(self, db, oid):
         self.oidcache = []
         offset = self._get_offset(oid)
-        rowref = self.get_iter(offset)
-        path = self.get_path(rowref)
-        self.row_deleted(path)
+        try:
+            rowref = self.get_iter(offset)
+            path = self.get_path(rowref)
+            self.row_deleted(path)
+        except ValueError:
+            #not a valid rowref
+            pass
 
     def _get_n_rows(self):
         """
@@ -100,7 +112,11 @@ class GenericDBListStore(gtk.GenericTreeModel):
         Parameters:
             offset -- the rows offset from 0.
         """
-        (oid,) = self.db.select_one("SELECT oid FROM %s LIMIT 1 OFFSET %d" % (self.table, offset))
+        try:
+            (oid,) = self.db.select_one("SELECT oid FROM %s LIMIT 1 OFFSET %d" % (self.table, offset))
+        except TypeError:
+            #Stops a crash at exit when the db is closed before the UI
+            oid = None
         return oid
     
     @DB.lru_cache(0)
