@@ -15,10 +15,6 @@ from conduit.ModuleWrapper import ModuleWrapper
 
 from gettext import gettext as _
 
-DND_TARGETS = [
-    ('conduit/element-name', 0, 0)
-    ]
-
 class CategoryWrapper(ModuleWrapper):
     """
     Represents a category stored in the treemodel. Not generally intended 
@@ -127,11 +123,9 @@ class DataProviderTreeModel(gtk.GenericTreeModel):
         built (in the constructor)
         @type signal: C{bool}
         """
-        log.debug("Adding DataProvider %s to TreeModel" % dpw)
         #Do we need to create a category first?
         i = self._get_category_index_by_name(dpw.category)
         if i == None:
-            log.debug("Creating Category: %s" % dpw.category.name)
             new_cat = CategoryWrapper(dpw.category)
             self.cats.append(new_cat)
             i = self.cats.index(new_cat)
@@ -208,7 +202,14 @@ class DataProviderTreeModel(gtk.GenericTreeModel):
                 return None
             if debug:
                 print "on_get_iter: path = %s cat = %s" % (path, self.cats[path[0]])
-            return self.cats[path[0]]
+            try:
+                return self.cats[path[0]]
+            except IndexError:
+                #I cannot reproducibly hit this code path. This bug just seems to occur
+                #on Ubuntu Lucid
+                #https://bugs.launchpad.net/bugs/506110
+                log.critical("Strange bug, cannot get iter...")
+                return None
         else:
             try:
                 if debug:
@@ -334,14 +335,18 @@ class DataProviderTreeView(gtk.TreeView):
     """
     Handles DND of DataProviders onto canvas
     """
+    
+    DND_TARGETS = [
+        ('conduit/element-name', 0, 0)
+    ]
+    
     def __init__(self, model):
         """
         Constructor
         """
         gtk.TreeView.__init__(self, model)
         self.set_property("enable-search", False)
-        #FIXME: Work around a (py)gtk 2.8 bug
-        if gtk.pygtk_version >= (2,10,0):
+        if gtk.pygtk_version >= (2,10,0) and conduit.GLOBALS.settings.get("gui_show_treeview_lines"):
             self.set_property("enable-tree-lines", True)
         
         #First column is an image and name
@@ -355,7 +360,7 @@ class DataProviderTreeView(gtk.TreeView):
         self.append_column(tvcolumn0)
 
         # Second column is a description
-        if conduit.GLOBALS.settings.get("show_dp_description") == True:
+        if conduit.GLOBALS.settings.get("show_dp_description"):
             tvcolumn1 = gtk.TreeViewColumn(_("Description"), gtk.CellRendererText(), text=IDX_DESCRIPTION)
             self.append_column(tvcolumn1)
             self.set_headers_visible(True)
@@ -365,11 +370,11 @@ class DataProviderTreeView(gtk.TreeView):
         # DND info:
         # drag
         self.enable_model_drag_source(  gtk.gdk.BUTTON1_MASK,
-                                        DND_TARGETS,
-                                        gtk.gdk.ACTION_DEFAULT | gtk.gdk.ACTION_MOVE)
-        self.drag_source_set(           gtk.gdk.BUTTON1_MASK | gtk.gdk.BUTTON3_MASK,
-                                        DND_TARGETS,
-                                        gtk.gdk.ACTION_COPY | gtk.gdk.ACTION_LINK)
+                               self.DND_TARGETS,
+                               gtk.gdk.ACTION_DEFAULT | gtk.gdk.ACTION_MOVE)
+        self.drag_source_set(  gtk.gdk.BUTTON1_MASK | gtk.gdk.BUTTON3_MASK,
+                               self.DND_TARGETS,
+                               gtk.gdk.ACTION_COPY | gtk.gdk.ACTION_LINK)
         self.connect('drag-data-get', self.on_drag_data_get)
         self.connect('drag-data-delete', self.on_drag_data_delete)
         
