@@ -14,21 +14,21 @@ from conduit.datatypes import Rid
 
 from gettext import gettext as _
 
-#We have to use our own flickrapi until the following is applied
-#http://sourceforge.net/tracker/index.php?func=detail&aid=1874067&group_id=203043&atid=984009
-Utils.dataprovider_add_dir_to_path(__file__)
-import flickrapi
+MODULES = {}
+try:
+    import flickrapi
+    if flickrapi.__version__ >= "1.2":
+        MODULES = {
+        	"FlickrTwoWay" :          { "type": "dataprovider" }        
+        }
+        log.info("Module Information: %s" % Utils.get_module_information(flickrapi, "__version__"))
+        #turn of debugging in the library
+        flickrapi.set_log_level(logging.NOTSET)
+except ImportError:
+    pass
 
-if flickrapi.__version__ == "1.1":
-    MODULES = {
-    	"FlickrTwoWay" :          { "type": "dataprovider" }        
-    }
-    log.info("Module Information: %s" % Utils.get_module_information(flickrapi, "__version__"))
-    #turn of debugging in the library
-    flickrapi.set_log_level(logging.NOTSET)
-else:
-    MODULES = {}
-    log.info("Flickr support disabled")
+if not MODULES:
+    log.info("Flickr support disabled. Please install python-flickrapi > 1.2")
     
 class MyFlickrAPI(flickrapi.FlickrAPI):
     """
@@ -279,38 +279,39 @@ class FlickrTwoWay(Image.ImageTwoWay):
         def _login_finished(*args):
             try:
                 if self.logged_in:
-                    status_label.value = _('Loading album list...')
+                    config['status'].value = _('Loading album list...')
                     try:
                         #FIXME: Blocks and brings the whole UI with it.
                         photosets = self._get_photosets()
                     except:
-                        status_label.value = _('Failed to connect.')
+                        config['status'].value = _('Failed to connect.')
                     else:
-                        photoset_config.choices = [name for name, photoSetId in photosets]
-                        status_label.value = _('Album names loaded.')
+                        config['photoSetName'].choices = [name for name, photoSetId in photosets]
+                        config['status'].value = _('Album names loaded.')
                 else:
-                    status_label.value = _('Failed to login.')
+                    config['status'].value = _('Failed to login.')
             finally:
-                load_photosets_config.enabled = True
+                config['photoSetName'].enabled = True
                 account_section.enabled = True
                 config.set_busy(False)
                 
         def _load_photosets(button):
             config.set_busy(True)
-            load_photosets_config.enabled = False
+            config['photoSetName'].enabled = False
             account_section.enabled = False
             username_config.apply()
-            status_label.value = _('Logging in, please wait...')
+            config['status'].value = _('Logging in, please wait...')
             conduit.GLOBALS.syncManager.run_blocking_dataprovider_function_calls(
                 self, _login_finished, self._login)
 
         account_section = config.add_section(_('Account details'))
-        username_config = config.add_item('Username', 'text',
+        username_config = config.add_item(_('Username'), 'text',
             config_name = 'username',
         )
         username_config.connect('value-changed',
             lambda item, initial, value: load_photosets_config.set_enabled(bool(value)))
         status_label = config.add_item(None, 'label',
+            name = 'status',
             initial_value = self.status,
             use_markup = True,
             xalignment = 0.5,
@@ -325,7 +326,7 @@ class FlickrTwoWay(Image.ImageTwoWay):
             choices = [],
         )
         config.add_item(_("Resize photos"), "combo",
-            choices = [("None", _("Do not resize")), "640x480", "800x600", "1024x768"],
+            choices = [("None", _("Do not resize")), _("640x480"), _("800x600"), _("1024x768")],
             config_name = "imageSize"
         )
         config.add_item(_('Photos are public'), 'check',
