@@ -10,6 +10,7 @@ import logging
 log = logging.getLogger("modules.N800")
 
 import conduit
+import conduit.datatypes.File as File
 import conduit.datatypes.Video as Video
 import conduit.datatypes.Audio as Audio
 import conduit.datatypes.Photo as Photo
@@ -52,12 +53,8 @@ class N800Base(FileDataProvider.FolderTwoWay):
     
     #Translators: Translate this in derived classes.
     DEFAULT_FOLDER = _("Conduit")
-    #Translators: Format string used to describe the acceptable formats the 
-    #device accepts. The first arg is replaced with DEFAULT_FOLDER and the second
-    #arg is a comma seperated list of encodings
-    FORMAT_CONVERSION_STRING = _("%s Format (%s)")
     #Signifies that a conversion should not take place
-    NO_CONVERSION_STRING = _("unchanged")
+    NO_CONVERSION_STRING = _("None")
 
     _configurable_ = True
 
@@ -72,31 +69,21 @@ class N800Base(FileDataProvider.FolderTwoWay):
         self.mount = mount
         self.udi = udi
         self.encodings =  {}
-        self.encoding = ""
+        self.update_configuration(
+            encoding = ""
+        )
 
-    def configure(self, window):
-        import gtk
-        import conduit.gtkui.SimpleConfigurator as SimpleConfigurator
-
-        def setEnc(param):
-            self.encoding = str(param)
-
-        encodings = self.encodings.keys()+[self.NO_CONVERSION_STRING]
-        items = [
-                    {
-                    "Name" : self.FORMAT_CONVERSION_STRING % (self.DEFAULT_FOLDER, ",".join(encodings)),
-                    "Widget" : gtk.Entry,
-                    "Callback" : setEnc,
-                    "InitialValue" : self.encoding
-                    }
-                ]
-        dialog = SimpleConfigurator.SimpleConfigurator(window, self._name_, items)
-        dialog.run()
+    def config_setup(self, config):
+        config.add_item(_("Encoding"), "radio",
+            config_name = "encoding",
+            choices = self.encodings.keys()+[self.NO_CONVERSION_STRING]
+        )
 
     def refresh(self):
-        if not Vfs.uri_exists(self.folder):
+        d = File.File(URI=self.folder)
+        if not d.exists():
             try:
-                Vfs.uri_make_directory_and_parents(self.folder)
+                d.make_directory_and_parents()
             except:
                 raise Exceptions.RefreshError("Error Creating Directory")
         FileDataProvider.FolderTwoWay.refresh(self)
@@ -107,9 +94,6 @@ class N800Base(FileDataProvider.FolderTwoWay):
         except KeyError:
             return {}
             
-    def get_configuration(self):
-        return {'encoding':self.encoding}
-
     def get_UID(self):
         return "%s" % self.udi
 
@@ -188,7 +172,7 @@ class N800PhotoTwoWay(N800Base):
     """
 
     _name_ = _("N800 Photos")
-    _description_ = _("Synchronizes video to a N800 device")
+    _description_ = _("Synchronizes photos to an N800 device")
     _in_type_ = "file/photo"
     _out_type_ = "file/photo"
     _icon_ = "image-x-generic"
