@@ -18,7 +18,7 @@ import socket
 import locale
 import weakref
 import threading
-DEFAULT_ENCODING = locale.getpreferredencoding()
+import gobject
 log = logging.getLogger("modules.iPod")
 
 import conduit
@@ -35,58 +35,78 @@ import conduit.datatypes.Video as Video
 
 from gettext import gettext as _
 
-MODULES = {
-    "iPodFactory" :         { "type":   "dataprovider-factory"  },
-}
-
+errormsg = ""
 try:
     import gpod
-    LIBGPOD_PHOTOS = gpod.version_info >= (0,6,0)
-    log.info("Module Information: %s" % Utils.get_module_information(gpod, 'version_info'))
+    if gpod.version_info >= (0,6,0):
+        MODULES = {
+            "iPodFactory" :         { "type":   "dataprovider-factory"  },
+        }
+        log.info("Module Information: %s" % Utils.get_module_information(gpod, 'version_info'))
 except ImportError:
-    LIBGPOD_PHOTOS = False
-    log.info("iPod photo support disabled")
+    errormsg = "iPod support disabled (python-gpod not availiable)"
+except locale.Error:
+    errormsg = "iPod support disabled (Incorrect locale)"
+
+if errormsg:
+    MODULES = {}
+    log.warn(errormsg)
+    #Solve the initialization problem without gpod
+    class gpod():
+        ITDB_MEDIATYPE_MUSICVIDEO = 0
+        ITDB_MEDIATYPE_MOVIE = 1
+        ITDB_MEDIATYPE_TVSHOW = 2
+        ITDB_MEDIATYPE_AUDIO = 3
+        ITDB_MEDIATYPE_PODCAST = 4
 
 def _string_to_unqiue_file(txt, base_uri, prefix, postfix=''):
-    for i in range(1, 10000):
-        filename = prefix + str(i) + postfix
-        uri = os.path.join(base_uri, filename)
-        f = File.File(uri)
-        if not f.exists():
-            break
-
     temp = Utils.new_tempfile(txt)
+    uri = os.path.join(base_uri, prefix+temp.get_filename()+postfix)
     temp.transfer(uri, True)
-    temp.set_UID(filename)
+    temp.set_UID(os.path.basename(uri))
     return temp.get_rid()
 
 class iPodFactory(VolumeFactory.VolumeFactory):
+
+    def _get_mount_path(self, props):
+        return str(props["volume.mount_point"])
+
     def is_interesting(self, udi, props):
-        if props.has_key("info.parent") and props.has_key("info.parent")!="":
-            prop2 = self._get_properties(props["info.parent"])
-            if prop2.has_key("storage.model") and prop2["storage.model"]=="iPod":
+        if props.get("info.parent"):
+            parent = self._get_properties(props["info.parent"])
+            if parent.get("storage.model") == "iPod":
+                props.update(parent)
                 return True
         return False
 
     def get_category(self, udi, **kwargs):
+        label = kwargs['volume.label']
+        if not label:
+            label = "Apple iPod Music Player"
         return DataProviderCategory.DataProviderCategory(
-                    kwargs['label'],
+                    label,
                     "multimedia-player-ipod-standard-color",
-                    kwargs['mount'])
+                    self._get_mount_path(kwargs))
 
     def get_dataproviders(self, udi, **kwargs):
-        if LIBGPOD_PHOTOS:
-            #Read information about the ipod, like if it supports
-            #photos or not
-            d = gpod.itdb_device_new()
-            gpod.itdb_device_set_mountpoint(d,kwargs['mount'])
-            supportsPhotos = gpod.itdb_device_supports_photo(d)
-            gpod.itdb_device_free(d)
-            if supportsPhotos:
-                return [IPodMusicTwoWay, IPodVideoTwoWay, IPodNoteTwoWay, IPodContactsTwoWay, IPodCalendarTwoWay, IPodPhotoSink]
-
-        return [IPodMusicTwoWay, IPodVideoTwoWay, IPodNoteTwoWay, IPodContactsTwoWay, IPodCalendarTwoWay]
+        #Read information about the ipod, like if it supports
+        #photos or not
+        d = gpod.itdb_device_new()
+        gpod.itdb_device_set_mountpoint(d,self._get_mount_path(kwargs))
+        supportsPhotos = gpod.itdb_device_supports_photo(d)
+        gpod.itdb_device_free(d)
+        if supportsPhotos:
+            return [IPodMusicTwoWay, IPodVideoTwoWay, IPodNoteTwoWay, IPodContactsTwoWay, IPodCalendarTwoWay, IPodPhotoSink]
+        else:
+            log.info("iPod does not report photo support")
+            return [IPodMusicTwoWay, IPodVideoTwoWay, IPodNoteTwoWay, IPodContactsTwoWay, IPodCalendarTwoWay]
 
+    def get_args(self, udi, **kwargs):
+        """
+        iPod needs a local path to the DB, not a URI
+        """
+        kwargs["mount_path"] = self._get_mount_path(kwargs)
+        return (kwargs['mount_path'], udi)
 
 class IPodBase(DataProvider.TwoWay):
     def __init__(self, *args):
@@ -95,6 +115,8 @@ class IPodBase(DataProvider.TwoWay):
         self.uid = args[1]
         self.objects = None
 
+        log.debug("Created ipod %s at %s" % (self.__class__.__name__, self.mountPoint))
+
     def refresh(self):
         DataProvider.TwoWay.refresh(self)
         self.objects = []
@@ -154,8 +176,8 @@ class IPodNoteTwoWay(IPodBase):
     LUID is the note title
     """
 
-    _name_ = "Notes"
-    _description_ = "Sync your iPod notes"
+    _name_ = _("Notes")
+    _description_ = _("Synchronize your iPod notes")
     _module_type_ = "twoway"
     _in_type_ = "note"
     _out_type_ = "note"
@@ -270,8 +292,8 @@ class IPodNoteTwoWay(IPodBase):
 
 class IPodContactsTwoWay(IPodBase):
 
-    _name_ = "Contacts"
-    _description_ = "Sync your iPod contacts"
+    _name_ = _("Contacts")
+    _description_ = _("Synchronize your iPod contacts")
     _module_type_ = "twoway"
     _in_type_ = "contact"
     _out_type_ = "contact"
@@ -306,8 +328,8 @@ class IPodContactsTwoWay(IPodBase):
 
 class IPodCalendarTwoWay(IPodBase):
 
-    _name_ = "Calendar"
-    _description_ = "Sync your iPod calendar"
+    _name_ = _("Calendar")
+    _description_ = _("Synchronize your iPod calendar")
     _module_type_ = "twoway"
     _in_type_ = "event"
     _out_type_ = "event"
@@ -342,8 +364,8 @@ class IPodCalendarTwoWay(IPodBase):
 
 class IPodPhotoSink(IPodBase):
 
-    _name_ = "Photos"
-    _description_ = "Sync your iPod photos"
+    _name_ = _("Photos")
+    _description_ = _("Synchronize your iPod photos")
     _module_type_ = "sink"
     _in_type_ = "file/photo"
     _out_type_ = "file/photo"
@@ -355,8 +377,10 @@ class IPodPhotoSink(IPodBase):
     def __init__(self, *args):
         IPodBase.__init__(self, *args)
         self.db = gpod.PhotoDatabase(self.mountPoint)
-        self.albumName = "Conduit"
         self.album = None
+        self.update_configuration(
+            albumName = "Conduit"
+        )
 
     def _set_sysinfo(self, modelnumstr, model):
         gpod.itdb_device_set_sysinfo(self.db._itdb.device, modelnumstr, model)
@@ -389,10 +413,11 @@ class IPodPhotoSink(IPodBase):
         if albumName == self.SAFE_PHOTO_ALBUM:
             log.warn("Cannot delete album: %s" % self.SAFE_PHOTO_ALBUM)
         else:
-            album = self._get_photo_album(albumName)
-            for photo in album[:]:
-                album.remove(photo)
-            self.db.remove(album)
+            for album in self.db.PhotoAlbums:
+                if album.name == albumName:
+                    for photo in album[:]:
+                        album.remove(photo)
+                    self.db.remove(album)
 
     def _empty_all_photos(self):
         for photo in self.db.PhotoAlbums[0][:]:
@@ -416,6 +441,7 @@ class IPodPhotoSink(IPodBase):
 
     def put(self, f, overwrite, LUID=None):
         photo = self.db.new_Photo(filename=f.get_local_uri())
+        self.album = self._get_photo_album(self.albumName)
         self.album.add(photo)
         gpod.itdb_photodb_write(self.db._itdb, None)
         return conduit.datatypes.Rid(str(photo['id']), None, hash(None))
@@ -426,61 +452,20 @@ class IPodPhotoSink(IPodBase):
             self.db.remove(photo)
             gpod.itdb_photodb_write(self.db._itdb, None)
 
-    def configure(self, window):
-        import gobject
-        import gtk
-        def build_album_model(albumCombo):
-            self.album_store.clear()
-            album_count = 0
-            album_iter = None
-            for name in self._get_photo_albums():
-                iter = self.album_store.append((name,))
-                if name == self.albumName:
-                    album_iter = iter
-                album_count += 1
+    def config_setup(self, config):
 
-            if album_iter:
-                albumCombo.set_active_iter(album_iter)
-            elif self.albumName:
-                albumCombo.child.set_text(self.albumName)
-            elif album_count:
-                albumCombo.set_active(0)
+        def _delete_click(button):
+            self._delete_album(album_config.get_value())
+            album_config.choices = self._get_photo_albums()
 
-        def delete_click(sender, albumCombo):
-            albumName = albumCombo.get_active_text()
-            if albumName:
-                self._delete_album(albumName)
-                build_album_model(albumCombo)
+        album_config = config.add_item(_('Album'), 'combotext',
+            config_name = 'albumName',
+            choices = self._get_photo_albums(),
+        )
+        config.add_item(_("Delete"), "button",
+            initial_value = _delete_click
+        )    
 
-        #get a whole bunch of widgets
-        tree = Utils.dataprovider_glade_get_widget(
-                        __file__,
-                        "config.glade",
-                        "PhotoConfigDialog")
-        albumCombo = tree.get_widget("album_combobox")
-        delete_button = tree.get_widget("delete_button")
-
-        #setup album store
-        self.album_store = gtk.ListStore(gobject.TYPE_STRING)
-        albumCombo.set_model(self.album_store)
-        cell = gtk.CellRendererText()
-        albumCombo.pack_start(cell, True)
-        albumCombo.set_text_column(0)
-
-        #setup widgets
-        build_album_model(albumCombo)
-        delete_button.connect('clicked', delete_click, albumCombo)
-
-        # run dialog
-        dlg = tree.get_widget("PhotoConfigDialog")
-        response = Utils.run_dialog(dlg, window)
-
-        if response == True:
-            #get the values from the widgets
-            self.albumName = albumCombo.get_active_text()
-        dlg.destroy()
-
-        del self.album_store
 
     def is_configured (self, isSource, isTwoWay):
         return len(self.albumName) > 0
@@ -488,93 +473,167 @@ class IPodPhotoSink(IPodBase):
     def uninitialize(self):
         self.db.close()
 
-STR_CONV = lambda v: unicode(v).encode('UTF-8','replace')
-INT_CONV = lambda v: int(v)
+unicode_conv = lambda v: unicode(v).encode('UTF-8','replace')
 
 class IPodFileBase:
-    # Supported tags from the Audio class supported in the iPod database
-    SUPPORTED_TAGS = ['title', 'artist', 'album', 'composer', 'rating',
-        'genre', 'track-number', 'track-count', 'bitrate', 'duration',
-        'samplerate']
-    # Conversion between Audio names and iPod names in tags
-    KEYS_CONV = {'duration': 'tracklen',
-                 'track-number': 'track_nr',
-                 'track-count': 'tracks'}
-    # Convert values into their native types
-    VALUES_CONV = {
-        'rating': lambda v: float(v) / 0.05,
-        'samplerate': INT_CONV,
-        'bitrate': INT_CONV,
-        'track-number': INT_CONV,
-        'track-count': INT_CONV,
-        'duration': INT_CONV,
-        'width': INT_CONV,
-        'height': INT_CONV
+    '''
+    A wrapper around an iPod track. iPod track properties are converted into
+    Media properties, and vice-versa.
+    '''
+    
+    #Mappings from the Media metadata to the iPod metadata and vice-versa, 
+    #including type-checking
+    media_to_ipod = {
+        'title' : ('title', unicode_conv),
+        'artist' : ('artist', unicode_conv),
+        'album' : ('album', unicode_conv),
+        'composer' : ('composer', unicode_conv),
+        'rating' : ('rating', lambda v: float(v) / 0.05),
+        'genre' : ('genre', unicode_conv),
+        'track_nr' : ('track-number', int),
+        'tracks' : ('track-count', int),
+        'bitrate' : ('bitrate', int),
+        'tracklen' : ('duration', int),
+        'samplerate' : ('samplerate', int),
+        'width' : ('width', int),
+        'height' : ('height', int),
+    }
+    
+    ipod_to_media = {
+        'title' : ('title', unicode_conv),
+        'artist' : ('artist', unicode_conv),
+        'album' : ('album', unicode_conv),
+        'composer' : ('composer', unicode_conv),
+        'rating' : ('rating', lambda v: float(v) * 0.05),
+        'genre' : ('genre', unicode_conv),
+        'track-number' : ('track_nr', int),
+        'track-count' : ('tracks', int),
+        'bitrate' : ('bitrate', int),
+        'duration' : ('tracklen', int),
+        'samplerate' : ('samplerate', int),
+        'width' : ('width', int),
+        'height' : ('height', int),        
     }
 
-    def __init__(self, db):
+    def __init__(self, db, track = None, f = None):
+        '''
+        Wraps an iPod track in a Datatype. 
+        Passing a file creates a new track in the iPod db, with media information 
+        from that file. Use copy_ipod to transfer it into the iPod.
+        Passing an existing iPod track exports the track's information as a 
+        Media datatype.
+        
+        @param ipod_track: An iPod track to wrap
+        @param f: A File to extract the information from
+        '''
         self.db = db
-        self.track = self.db.new_Track()
+        if track:
+            self.track = track
+        else:
+            self.track = self.db.new_Track()
+        if f:
+            self.set_info_from_file(f)
+        
+    def get_UID(self):
+        '''
+        Returns the database ID (usually a random number, which is always valid
+        for this track in this db, even across application restarts)
+        '''
+        return str(self.track['dbid'])
+
+    def _convert_tags(self, from_tags, mapping):
+        '''
+        Convert from one mapping to another.
+        Returns an iterator with (name, value) for each tag in from_tags
+        '''
+        for from_name, from_value in from_tags.iteritems():
+            if from_name in mapping:
+                to_name, to_converter = mapping[from_name]
+                try:
+                    to_value = to_converter(from_value)
+                    yield to_name, to_value
+                except Exception, e:
+                    log.warn("Could not convert property %s: %s as %s. (Error: %s)" % (from_name, from_value, to_converter, e))
 
     def set_info_from_file(self, f):
-        # Missing: samplerate (int), samplerate2 (float), bitrate (int),
-        # composer (str), filetype (str, "MPEG audio file"), mediatype (int, 1)
-        # tracks (int)
-        # unk126 (int, "65535"), unk144 (int, "12"),
+        '''
+        Get the track information from a file, including the metadata.
+        Works best with GStreamer metadata in MediaFile.
+        '''
         tags = f.get_media_tags()
-        for key, value in tags.iteritems():
-            if key not in self.SUPPORTED_TAGS:
-                continue
-            if key in self.VALUES_CONV:
-                # Convert values into nativa types
-                tag_value = self.VALUES_CONV[key](value)
-            else:
-                # Encode into UTF-8
-                tag_value = STR_CONV(value)
-            if key in self.KEYS_CONV:
-                tag_name = self.KEYS_CONV[key]
-            else:
-                tag_name = key
-            self.track[tag_name] = tag_value
-        print self.track['title']
+        for name, value in self._convert_tags(tags, self.media_to_ipod):
+            #log.debug("Got %s = %s" % (name, value))
+            self.track[name] = value
+        #Make sure we have a title to this song, even if it's just the filename
         if self.track['title'] is None:
             self.track['title'] = os.path.basename(f.get_local_uri())
-            print self.track['title']
         self.track['time_modified'] = os.stat(f.get_local_uri()).st_mtime
         self.track['time_added'] = int(time.time())
         self.track['userdata'] = {'transferred': 0,
                                   'hostname': socket.gethostname(),
-                                  'charset': DEFAULT_ENCODING}
+                                  'charset': locale.getpreferredencoding()}
         self.track._set_userdata_utf8('filename', f.get_local_uri())
+        
+    def get_track_filename(self):
+        filename = self.track.ipod_filename()
+        if not filename or not os.path.exists(filename):
+            filename = self.track._userdata_into_default_locale('filename')
+        return filename
+        
+    def get_hash(self):
+        return str(hash(tuple(self.get_media_tags())))
+
+    def get_snippet(self):
+        return "%(artist)s - %(title)s" % track
+
+    def get_media_tags(self):
+        '''
+        Extends the MediaFile class to include the iPod metadata, instead of
+        calling the GStreamer loader. It's much faster this way, and provides
+        some nice information to other dataproviders, like ratings.
+        '''
+        #FIXME: Cache this information
+        
+        #Get the information from the iPod track.
+        #The track might look like a dict, but it isnt, so we make it into one.
+        track_tags = dict(self.track.pairs())
+        return dict(self._convert_tags(track_tags, self.ipod_to_media))
 
     #FIXME: Remove this. Use native operations from Conduit instead.
+    #       We would have to define the transfered userdata as 1 and then call
+    #       Conduit to copy the file. 
+    #       But that is Conduit's copy file way?
     def copy_ipod(self):
         self.track.copy_to_ipod()
 
-class IPodAudio(Audio.Audio, IPodFileBase):
-    def __init__(self, f, db, **kwargs):
-        Audio.Audio.__init__(self, f.get_local_uri())
-        IPodFileBase.__init__(self, db)
-        self.set_info_from_audio(f)
+class IPodAudio(IPodFileBase, Audio.Audio):
+    def __init__(self, *args, **kwargs):
+        '''
+        Initialize a new Audio track for this db and file.
+        '''
+        IPodFileBase.__init__(self, *args, **kwargs)
+        Audio.Audio.__init__(self, URI = self.get_track_filename())
 
-    def set_info_from_audio(self, audio):
-        self.set_info_from_file(audio)
+    def set_info_from_file(self, audio):
+        IPodFileBase.set_info_from_file(self, audio)
         self.track['mediatype'] = gpod.ITDB_MEDIATYPE_AUDIO
         cover_location = audio.get_audio_cover_location()
         if cover_location:
             self.track.set_coverart_from_file(str(cover_location))
 
-class IPodVideo(Video.Video, IPodFileBase):
-    def __init__(self, f, db, **kwargs):
-        Video.Video.__init__(self, f.get_local_uri())
-        IPodFileBase.__init__(self, db)
+class IPodVideo(IPodFileBase, Video.Video):
+    def __init__(self, *args, **kwargs):
+        '''
+        Initialize a new Video track for this db and file.
+        '''
+        IPodFileBase.__init__(self, *args, **kwargs)
+        Video.Video.__init__(self, URI = self.get_track_filename())
+        
         log.debug('Video kind selected: %s' % (kwargs['video_kind']))
         self.video_kind = kwargs['video_kind']
-        self.set_info_from_video(f)
-
-    def set_info_from_video(self, video):
-        self.set_info_from_file(video)
-        #FIXME: Movie should be a choice between Movie, MusicVideo, TvShow and Podcast
+        
+    def set_info_from_file(self, video):
+        IPodFileBase.set_info_from_file(video)
         self.track['mediatype'] = {'movie': gpod.ITDB_MEDIATYPE_MOVIE,
                                    'musicvideo': gpod.ITDB_MEDIATYPE_MUSICVIDEO,
                                    'tvshow': gpod.ITDB_MEDIATYPE_TVSHOW,
@@ -599,7 +658,6 @@ class DBCache:
             if mount_point in self.__db_list:
                 log.debug('Getting DB in cache for %s' % (mount_point))
                 db = self.__db_list[mount_point]
-                #self.__db_locks[db][1] += 1
             else:
                 if mount_point:
                     log.debug('Creating DB for %s' % mount_point)
@@ -616,15 +674,14 @@ class DBCache:
     @classmethod
     def release_db(self, db):
         assert db in self.__db_locks
+        # We dont do nothing here yet, but we could use to release resources.
+        # The db is automatically removed from the list because of the weak 
+        # reference.
         log.debug('Releasing DB for %s' % db)
-        #self.__db_locks[db][1] -= 1
 
     @classmethod
     def lock_db(self, db):
         assert db in self.__db_locks
-        #if self.__db_locks[db][1] == 1:
-        #    log.debug('Not locking DB for %s' % db)
-        #    return
         log.debug('Locking DB %s' % db)
         self.__db_locks[db].acquire()
 
@@ -632,69 +689,99 @@ class DBCache:
     def unlock_db(self, db):
         assert db in self.__db_locks
         log.debug('Unlocking DB %s' % db)
-        #lock = self.__db_locks[db][0]
-        #if lock.locked():
         self.__db_locks[db].release()
 
 class IPodMediaTwoWay(IPodBase):
     FORMAT_CONVERSION_STRING = _("Encoding")
 
     def __init__(self, *args):
-        if len(args) != 0:
+        self.local_db = (len(args) == 0)
+        if not self.local_db:
             IPodBase.__init__(self, *args)
-            self.db = DBCache.get_db(self.mountPoint)
         else:
             # Use local database for testing
-            DataProvider.TwoWay.__init__(self)
-            self.db = DBCache.get_db(None)
+            DataProvider.TwoWay.__init__(self)        
             self.uid = "Local"
+        self.db = None
         #self.tracks = {}
         self.tracks_id = {}
         self.track_args = {}
+        self.update_configuration(
+            keep_converted = True,
+        )
+        
+    def get_db(self):
+        if self.db:
+            DBCache.lock_db(self.db)
+            return self.db
+        if not self.local_db:
+            self.db = DBCache.get_db(self.mountPoint)
+        else:
+            self.db = DBCache.get_db(None)        
+        DBCache.lock_db(self.db)
+        return self.db
+    
+    def unlock_db(self):
+        DBCache.unlock_db(self.db)
+        
+    def release_db(self):
+        if not self.db:
+            return
+        self.db.close()
+        DBCache.release_db(self.db)
+        self.db = None        
 
     def refresh(self):
         DataProvider.TwoWay.refresh(self)
         self.tracks = {}
         self.tracks_id = {}
-        DBCache.lock_db(self.db)
+        self.get_db()
         try:
             def add_track(track):
-                self.tracks_id[track['dbid']] = track
-                #FIXME: We dont need this do we?
-                #self.tracks[(track['artist'], track['title'])] = track
+                self.tracks_id[str(track['dbid'])] = track
             [add_track(track) for track in self.db \
                 if track['mediatype'] in self._mediatype_]
         finally:
-            DBCache.unlock_db(self.db)
+            self.unlock_db()
 
     def get_all(self):
         return self.tracks_id.keys()
 
     def get(self, LUID = None):
-        DBCache.lock_db(self.db)
+        self.get_db()
         try:
+            if LUID not in self.tracks_id:
+                raise Exceptions.SyncronizeError('Track ID %s not found in iPod DB %s' % (LUID, self.db))
             track = self.tracks_id[LUID]
-            if track and track.ipod_filename() and os.path.exists(track.ipod_filename()):
-                f = self._mediafile_(URI=track.ipod_filename())
-                f.set_UID(LUID)
-                f.set_open_URI(track.ipod_filename())
-                if track['artist'] and track['title']:
-                    f.force_new_filename("%(artist)s - %(title)s" % track + \
-                        os.path.splitext(track.ipod_filename())[1])
-                return f
+            ipod_file = self._ipodmedia_(self.db, track = track)
+            filename = ipod_file.get_track_filename()
+            if not os.path.exists(filename):
+                raise Exceptions.SyncronizeError("Could not find iPod track file %s" % (filename))
+            #Set a nice "Artist - Title" name with the original filename
+            #extension
+            #FIXME: Doesnt work as expected anymore, the original filename is
+            #renamed instead
+            #if track.ipod_filename() and track['artist'] and track['title']:
+            #    ipod_file.force_new_filename("%(artist)s - %(title)s" % track + \
+            #        os.path.splitext(filename)[1])
+            return ipod_file
         finally:
-            DBCache.unlock_db(self.db)
+            self.unlock_db()
         return None
 
     def put(self, f, overwrite, LUID=None):
-        DBCache.lock_db(self.db)
+        self.get_db()
         try:
-            media_file = self._ipodmedia_(f, self.db, **self.track_args)
-            #FIXME: We keep the db locked while we copy the file. Not good
+            if LUID and LUID in self.tracks_id:
+                track = self.tracks_id[LUID]
+                media_file = self._ipodmedia_(db = self.db, track = track, f = f, **self.track_args)
+            else:
+                media_file = self._ipodmedia_(db = self.db, f = f, **self.track_args)
+            #FIXME: We keep the db locked while we copy the file. Not good.
             media_file.copy_ipod()
+            self.tracks_id[str(media_file.track['dbid'])] = media_file.track
             #FIXME: Writing the db here is for debug only. Closing does not actually
-            # close the db, it only writes it's contents to disk.
-            
+            # close the db, it only writes it's contents to disk.            
             # Sometimes, if we only close the db when the sync is over, it might
             # take a long time to close the db, because many files are being 
             # copied to the iPod. Closing the DB every time not only keeps
@@ -704,78 +791,60 @@ class IPodMediaTwoWay(IPodBase):
             self.db.close()
             return media_file
         finally:
-            DBCache.unlock_db(self.db)
+            self.unlock_db()
 
     def delete(self, LUID):
         track = self.tracks_id[LUID]
         if track:
-            DBCache.lock_db(db)
+            self.get_db()
             try:
                 self.db.remove(track)
                 self.db.close()
             finally:
-                DBCache.unlock_db(db)
+                self.unlock_db()
 
-    def get_config_items(self):
-        import gtk
-        def dict_update(a, b):
-            a.update(b)
-            return a
+    def config_setup(self, config):
         #Get an array of encodings, so it can be indexed inside a combobox
-        self.config_encodings = [dict_update({'name': name}, value) for name, value in self.encodings.iteritems()]
-        initial = None
-        for encoding in self.config_encodings:
-            if encoding['name'] == self.encoding:
-                initial = encoding.get('description', None) or encoding['name']
-
-        def selectEnc(index, text):
-            self.encoding = self.config_encodings[index]['name']
-            log.debug('Encoding %s selected' % self.encoding)
-            
-        return [
-                    {
-                    "Name" : self.FORMAT_CONVERSION_STRING,
-                    "Kind" : "list",
-                    "Callback" : selectEnc,
-                    "Values" : [encoding.get('description', None) or encoding['name'] for encoding in self.config_encodings],
-                    "InitialValue" : initial
-                    }
-                ]        
-
-    def configure(self, window):
-        import conduit.gtkui.SimpleConfigurator as SimpleConfigurator
-
-        dialog = SimpleConfigurator.SimpleConfigurator(window, self._name_, self.get_config_items())
-        dialog.run()
-
-    def set_configuration(self, config):
-        if 'encoding' in config:
-            self.encoding = config['encoding']
-
-    def get_configuration(self):
-        return {'encoding':self.encoding}
+        encodings = [(enc_name, enc_opts.get('description', None) or enc_name)
+                      for enc_name, enc_opts in self.encodings.iteritems()]
+        
+        config.add_section(_("Conversion options"))
+        config.add_item(_("Encoding"), "combo", 
+            config_name = "encoding",
+            choices = encodings
+        )
+        config.add_item(_("Keep converted files"), "check",
+            config_name = "keep_converted"
+        )
 
     def get_input_conversion_args(self):
         try:
-            return self.encodings[self.encoding]
+            args = self.encodings[self.encoding]
+            # FIXME
+            # If we pass the bool in the args, it will become a string, and 
+            # will always return True later in the converter.
+            # So we only pass it if is True. When it's False, not being there
+            # tells the converter it isn't True.
+            # I'm not sure it was supposed to work like this.
+            if self.keep_converted:
+                args['keep_converted'] = True
+            return args
         except KeyError:
             return {}
 
     def uninitialize(self):
-        self.db.close()
-        DBCache.release_db(self.db)
-        self.db = None
+        self.release_db()
 
 IPOD_AUDIO_ENCODINGS = {
     "mp3": {"description": "Mp3", "acodec": "lame", "file_extension": "mp3"},
-    #FIXME: Does AAC needs a MP4 mux?
-    "aac": {"description": "AAC", "acodec": "faac", "file_extension": "m4a"},
+    #FIXME: AAC needs a MP4 mux
+    #"aac": {"description": "AAC", "acodec": "faac", "file_extension": "m4a"},
     }
 
 class IPodMusicTwoWay(IPodMediaTwoWay):
 
-    _name_ = "Music"
-    _description_ = "Sync your iPod music"
+    _name_ = _("iPod Music")
+    _description_ = _("Synchronize your iPod music")
     _module_type_ = "twoway"
     _in_type_ = "file/audio"
     _out_type_ = "file/audio"
@@ -783,77 +852,67 @@ class IPodMusicTwoWay(IPodMediaTwoWay):
     _configurable_ = True
 
     _mediatype_ = (gpod.ITDB_MEDIATYPE_AUDIO,)
-    _mediafile_ = Audio.Audio
     _ipodmedia_ = IPodAudio
 
     def __init__(self, *args):
         IPodMediaTwoWay.__init__(self, *args)
         self.encodings = IPOD_AUDIO_ENCODINGS
-        self.encoding = 'aac'
+        self.update_configuration(
+            encoding = 'mp3',
+        )
 
 IPOD_VIDEO_ENCODINGS = {
-    "mp4_x264":{"description": "MP4 (H.264)","vcodec":"x264enc", "acodec":"faac", "format":"ffmux_mp4", "file_extension":"m4v", "width": 320, "height": 240},
-    "mp4_xvid":{"description": "MP4 (XVid)","vcodec":"xvidenc", "acodec":"faac", "format":"ffmux_mp4", "file_extension":"m4v", "width": 320, "height": 240},
+    #FIXME: Add iPod mpeg4 restrictions. Follow:
+    # http://rob.opendot.cl/index.php/useful-stuff/ffmpeg-x264-encoding-guide/
+    "mp4_x264":{"description": "MP4 (Better quality - H.264)","vcodec":"x264enc", "acodec":"faac", 
+        "format":"ffmux_mp4", "file_extension":"m4v", "width": 320, "height": 240, 
+        "mimetype": "video/mp4"},
+    #FIXME: Two-pass encoding is not working. The first pass never finishes.
+    #"mp4_x264_twopass":{"description": "MP4 (H.264, Two-pass EXPERIMENTAL)", 
+    #    "vcodec_pass1":"x264enc pass=1", "vcodec_pass2":"x264enc pass=2", 
+    #    "acodec":"faac", "format":"ffmux_mp4", "file_extension":"m4v", 
+    #    "width": 320, "height": 240, "mimetype": "video/mp4", 'twopass':True},
+    "mp4_xvid":{"description": "MP4 (Faster conversion - XVid)","vcodec":"ffenc_mpeg4", "acodec":"faac",
+        "format":"ffmux_mp4", "file_extension":"m4v", "width": 320, "height": 240, 
+        "mimetype": "video/mp4"},
     }
 
 class IPodVideoTwoWay(IPodMediaTwoWay):
 
-    _name_ = "Video"
-    _description_ = "Sync your iPod videos"
+    _name_ = _("iPod Video")
+    _description_ = _("Synchronize your iPod videos")
     _module_type_ = "twoway"
     _in_type_ = "file/video"
     _out_type_ = "file/video"
     _icon_ = "video-x-generic"
     _configurable_ = True
 
-    _mediatype_ = (gpod.ITDB_MEDIATYPE_MUSICVIDEO,
-                   gpod.ITDB_MEDIATYPE_MOVIE,
-                   gpod.ITDB_MEDIATYPE_TVSHOW)
-    _mediafile_ = Video.Video
+    _mediatype_ = (gpod.ITDB_MEDIATYPE_MUSICVIDEO, gpod.ITDB_MEDIATYPE_MOVIE, gpod.ITDB_MEDIATYPE_TVSHOW)
     _ipodmedia_ = IPodVideo
 
     def __init__(self, *args):
         IPodMediaTwoWay.__init__(self, *args)
         self.encodings = IPOD_VIDEO_ENCODINGS
-        self.encoding = 'mp4_x264'
-        self.video_kind = 'movie'
+        self.update_configuration(
+            encoding = 'mp4_x264',
+            video_kind = 'movie',
+        )
         self._update_track_args()
         
     def _update_track_args(self):
         self.track_args['video_kind'] = self.video_kind
 
-    def get_config_items(self):
-        video_kinds = [('Movie', 'movie'), 
-                       ('Music Video', 'musicvideo'),
-                       ('TV Show', 'tvshow')]
-        initial = None
-        for description, name in video_kinds:
-            if name == self.video_kind:
-                initial = description
-
-        def selectKind(index, text):
-            self.video_kind = video_kinds[index][1]
-            self._update_track_args()
-
-        items = IPodMediaTwoWay.get_config_items(self)
-        items.append( 
-                        {
-                            "Name" : "Video Kind",
-                            "Kind" : "list",
-                            "Callback" : selectKind,
-                            "Values" : [description for description, name in video_kinds],
-                            "InitialValue" : initial
-                        } 
-                    )             
-                    
-        return items
-    
+    def config_setup(self, config):
+        IPodMediaTwoWay.config_setup(self, config)
+        video_kinds = [('movie', _('Movie')), 
+                       ('musicvideo', _('Music Video')),
+                       ('tvshow', _('TV Show'))]            
+        config.add_section()
+        config.add_item(_("Video kind"), "combo",
+            config_name = "video_kind",
+            choices = video_kinds)
+        
     def set_configuration(self, config):
         IPodMediaTwoWay.set_configuration(self, config)
-        if 'video_kind' in config:
-            self.encoding = config['video_kind']
-
-    def get_configuration(self):
-        config = IPodMediaTwoWay.get_configuration(self)
-        config.update({'encoding':self.encoding})
-        return config
+        #FIXME Move this to update_configuration callback
+        self._update_track_args()
